import { $$, forEach } from "@acanto/core-dom";

/**
 * Prepare (private behaviour)
 * 
 * @param {import("@barba/core/dist/core/src/defs/transition").ITransitionData} data Barba hook data
 */
function swapRoutesStyle(data) {
  const nextId = data.next.namespace;
  if (__DEV__) {
    console.info("routesBarba: swapRoutesStyle, next id is:" + nextId);
  }

  // enable only the next route's style tag
  forEach($$("[data-route-style]"), $style => {
    if ($style.getAttribute("data-route-style") === nextId) {
      if (__DEV__) console.info("routesBarba: enabling style for" + nextId);
      $style.removeAttribute("type");
    } else {
      if (__DEV__) console.info("routesBarba: disabling style for" + $style.id);
      $style.setAttribute("type", "text");
    }
  });
}

/**
 * Routes definitions for barba-js
 *
 * This file is autogenerated, the idea is to keep having a per route isolation
 * both interms of JS entrypoints and CSS inlined per route. The latter
 * is simply managed through inlining the new route's CSS inside the
 * asynchronously fetched new route html and disabling the old one `beforEnter`.
 * Each of these CSS are `<style>` tag with a conventional id like
 * `__route-style-${id}` and generically matched by the data attr `route-style`.
 * Per route JS handling is a bit more complicated, and the reason for the
 * existence of this autogenerated file. Webpack async importa cannot have
 * a variable in their path, hence we precompile this file with the routes
 * information returned by the CMS during the build step in the CI (and on `npm
 * run start` too). We build a barba view for each of our routes, attaching the
 * barba hooks to standard lifecycle methods of the routes' entrypoints
 * components:
 * `beforeEnter` => `route()` (basic component instantiation),
 * `afterEnter` => `route.mount()`,
 * `afterLeave` => `route.destroy()`
 * 
 * For some of this topics see this github issue
 * @see https://github.com/barbajs/barba/issues/410
 * where it suggested to just create a single js/css file, that might also work
 * if we automatically scope the css on the entire page. But at the moment, with
 * the current approach we are free to keep having global styles applied per
 * route without having to worry of style collisions, for instance we can have
 * each route specifying a body background without having to scope the css
 * selector and that's something useful and comfortable to use. Also JS might
 * become very heavy if we just pack everything in a single file...
 */
export default function routesBarba() {
  const instances = {};

  function getInstanceSafely(id, module) {
    if (!instances[id]) {
      // class based routes:
      // instances[id] = new module.default({ id });
      // function based routes:
      instances[id] = module.default({ id });
    }
    return instances[id];
  }

  return [<% routes.forEach(route => { %>{
      namespace: "<%= route.id %>",
      beforeEnter(data) {
        if (__DEV__) {
          console.log("routesBarba: <%= route.id %>->beforeEnter", data);
        }
        import("routes/<%= route.id %>").then((module) => {
          getInstanceSafely("<%= route.id %>", module);
        });
        
        swapRoutesStyle(data);
      },
      afterEnter(data) {
        if (__DEV__) {
          console.log("routesBarba: <%= route.id %>->afterEnter", data);
          // import scss too during dev as we are using style-loader...
          import("routes/<%= route.id %>/index.scss");
        }
        import("routes/<%= route.id %>").then((module) => {
          getInstanceSafely("<%= route.id %>", module).__m(data);
        });
      },
      afterLeave(data) {
        if (__DEV__) {
          console.log("routesBarba: <%= route.id %>->afterLeave", data);
        }
        import("routes/<%= route.id %>").then((module) => {
          getInstanceSafely("<%= route.id %>", module).__d(data)
        });
      }
    },<% }) %>  
  ];
}
